@@ -1,18 +1,19 @@
-import os
+import tempfile
 from copy import deepcopy
 from collections import UserDict
 import jsonpickle
 from argo.workflows.client import (
     V1alpha1Inputs,
     V1alpha1Outputs,
-    V1alpha1Parameter,
     V1alpha1RawArtifact,
     V1alpha1ArchiveStrategy
 )
-from .client import V1alpha1ValueFrom, V1alpha1Artifact
+from .client import V1alpha1ValueFrom, V1alpha1Parameter, V1alpha1Artifact
 from .common import S3Artifact
 from .utils import upload_s3, randstr
 
+NotAllowedInputArtifactPath = ["/", "/tmp"]
+
 class AutonamedDict(UserDict):
     def __setitem__(self, key, value):
         value = deepcopy(value)
@@ -20,13 +21,14 @@ class AutonamedDict(UserDict):
         super().__setitem__(key, value)
 
 class Inputs:
+    """
+    Inputs for OP template
+
+    Args:
+        paramters: input parameters
+        artifacts: input artifacts
+    """
     def __init__(self, parameters=None, artifacts=None):
-        """
-        Inputs for OP template
-        :param paramters: input parameters
-        :param artifacts: input artifacts
-        :return:
-        """
         if parameters is not None:
             self.parameters = parameters
         else:
@@ -62,13 +64,14 @@ class Inputs:
         return V1alpha1Inputs(parameters=parameters, artifacts=artifacts)
 
 class Outputs:
+    """
+    Outputs for OP template
+
+    Args:
+        paramters: output parameters
+        artifacts: output artifacts
+    """
     def __init__(self, parameters=None, artifacts=None):
-        """
-        Outputs for OP template
-        :param paramters: output parameters
-        :param artifacts: output artifacts
-        :return:
-        """
         if parameters is not None:
             self.parameters = parameters
         else:
@@ -149,15 +152,16 @@ class ArgoVar:
         return ArgoVar("asFloat(%s) >= %s" % (self.expr, other))
 
 class InputParameter(ArgoVar):
+    """
+    Input parameter for OP template
+
+    Args:
+        name: name of the input parameter
+        step_id:
+        type: parameter type
+        value: default value
+    """
     def __init__(self, name=None, step_id=None, type=None, value=None, save_as_artifact=False, path=None, source=None):
-        """
-        Input parameter for OP template
-        :param name: name of the input parameter
-        :param step_id:
-        :param type: parameter type
-        :param value: default value
-        :return:
-        """
         self.name = name
         self.step_id = step_id
         self.type = type
@@ -171,8 +175,8 @@ class InputParameter(ArgoVar):
             if self.save_as_artifact:
                 if self.name is not None:
                     if self.step_id is not None:
-                        return "steps['%s'].inputs.artifacts['%s']" % (self.step_id, self.name)
-                    return "inputs.artifacts['%s']" % self.name
+                        return "steps['%s'].inputs.artifacts['dflow_bigpar_%s']" % (self.step_id, self.name)
+                    return "inputs.artifacts['dflow_bigpar_%s']" % self.name
                 return ""
             if self.name is not None:
                 if self.step_id is not None:
@@ -185,8 +189,8 @@ class InputParameter(ArgoVar):
         if self.save_as_artifact:
             if self.name is not None:
                 if self.step_id is not None:
-                    return "{{steps.%s.inputs.artifacts.%s}}" % (self.step_id, self.name)
-                return "{{inputs.artifacts.%s}}" % self.name
+                    return "{{steps.%s.inputs.artifacts.dflow_bigpar_%s}}" % (self.step_id, self.name)
+                return "{{inputs.artifacts.dflow_bigpar_%s}}" % self.name
             return ""
         if self.name is not None:
             if self.step_id is not None:
@@ -195,46 +199,62 @@ class InputParameter(ArgoVar):
         return ""
 
     def convert_to_argo(self):
+        description = None
+        if self.type is not None:
+            try:
+                description = jsonpickle.dumps({"type": self.type})
+            except:
+                description = jsonpickle.dumps({"type": None})
+
         if self.save_as_artifact:
             if self.value is not None:
-                path = ".tmp-%s" % self.name
-                with open(path, "w") as f:
+                with tempfile.TemporaryDirectory() as tmpdir:
+                    content = {}
                     if isinstance(self.value, str):
-                        f.write(self.value)
+                        content["value"] = self.value
                     else:
-                        f.write(jsonpickle.dumps(self.value))
-                key = upload_s3(path)
-                s3 = S3Artifact(key=key)
-                os.remove(path)
-                return V1alpha1Artifact(name=self.name, path=self.path, s3=s3)
+                        content["value"] = jsonpickle.dumps(self.value)
+                    if self.type is not None:
+                        try:
+                            jsonpickle.dumps(self.type)
+                            content["type"] = self.type
+                        except:
+                            content["type"] = None
+                    path = tmpdir + "/" + self.name
+                    with open(path, "w") as f:
+                        f.write(jsonpickle.dumps(content))
+                    key = upload_s3(path)
+                    s3 = S3Artifact(key=key)
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, path=self.path, s3=s3)
             elif isinstance(self.source, (InputParameter, OutputParameter)):
-                return V1alpha1Artifact(name=self.name, path=self.path, _from=str(self.source))
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, path=self.path, _from=str(self.source))
             elif isinstance(self.source, (InputArtifact, OutputArtifact)):
-                return V1alpha1Artifact(name=self.name, path=self.path, _from=str(self.source))
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, path=self.path, _from=str(self.source))
             else:
-                return V1alpha1Artifact(name=self.name, path=self.path)
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, path=self.path)
 
         if self.value is None:
-            return V1alpha1Parameter(name=self.name)
+            return V1alpha1Parameter(name=self.name, description=description)
         elif isinstance(self.value, ArgoVar):
-            return V1alpha1Parameter(name=self.name, value="{{=%s}}" % self.value.expr)
+            return V1alpha1Parameter(name=self.name, value="{{=%s}}" % self.value.expr, description=description)
         elif isinstance(self.value, str):
-            return V1alpha1Parameter(name=self.name, value=self.value)
+            return V1alpha1Parameter(name=self.name, value=self.value, description=description)
         else:
-            return V1alpha1Parameter(name=self.name, value=jsonpickle.dumps(self.value))
+            return V1alpha1Parameter(name=self.name, value=jsonpickle.dumps(self.value), description=description)
 
 class InputArtifact(ArgoVar):
+    """
+    Input artifact for OP template
+
+    Args:
+        path: path where the input artifact is placed in the container
+        name: name of the input artifact
+        step_id:
+        optional: optional artifact or not
+        type: artifact type
+        source: default source
+    """
     def __init__(self, path=None, name=None, step_id=None, optional=False, type=None, source=None):
-        """
-        Input artifact for OP template
-        :param path: path where the input artifact is placed in the container
-        :param name: name of the input artifact
-        :param step_id:
-        :param optional: optional artifact or not
-        :param type: artifact type
-        :param source: default source
-        :return:
-        """
         self.path = path
         self.name = name
         self.step_id = step_id
@@ -265,6 +285,8 @@ class InputArtifact(ArgoVar):
         return artifact
 
     def convert_to_argo(self):
+        if self.path in NotAllowedInputArtifactPath:
+            raise RuntimeError("Path [%s] is not allowed for input artifact" % self.path)
         if self.source is None:
             return V1alpha1Artifact(name=self.name, path=self.path, optional=self.optional)
         if isinstance(self.source, (InputArtifact, OutputArtifact)):
@@ -277,20 +299,21 @@ class InputArtifact(ArgoVar):
             raise RuntimeError("Cannot pass an object of type %s to artifact %s" % (type(self.source)), self)
 
 class OutputParameter(ArgoVar):
+    """
+    Output parameter for OP template
+
+    Args:
+        value_from_path: the value is read from file generated in the container
+        value_from_parameter: the value is from another parameter
+        name: name of the output parameter
+        step_id:
+        type: parameter type
+        default: default value
+        global_name: global name of the parameter within the workflow
+        value_from_expression: the value is from an expression
+    """
     def __init__(self, value_from_path=None, value_from_parameter=None, name=None, step_id=None, type=None, default=None, global_name=None,
             value_from_expression=None, save_as_artifact=False):
-        """
-        Output parameter for OP template
-        :param value_from_path: the value is read from file generated in the container
-        :param value_from_parameter: the value is from another parameter
-        :param name: name of the output parameter
-        :param step_id:
-        :param type: parameter type
-        :param default: default value
-        :param global_name: global name of the parameter within the workflow
-        :param value_from_expression: the value is from an expression
-        :return:
-        """
         self.value_from_path = value_from_path
         self.value_from_parameter = value_from_parameter
         self.name = name
@@ -306,8 +329,8 @@ class OutputParameter(ArgoVar):
             if self.save_as_artifact:
                 if self.name is not None:
                     if self.step_id is not None:
-                        return "steps['%s'].outputs.artifacts['%s']" % (self.step_id, self.name)
-                    return "outputs.artifacts['%s']" % self.name
+                        return "steps['%s'].outputs.artifacts['dflow_bigpar_%s']" % (self.step_id, self.name)
+                    return "outputs.artifacts['dflow_bigpar_%s']" % self.name
             if self.name is not None:
                 if self.step_id is not None:
                     return "steps['%s'].outputs.parameters['%s']" % (self.step_id, self.name)
@@ -327,8 +350,8 @@ class OutputParameter(ArgoVar):
         if self.save_as_artifact:
             if self.name is not None:
                 if self.step_id is not None:
-                    return "{{steps.%s.outputs.artifacts.%s}}" % (self.step_id, self.name)
-                return "{{outputs.artifacts.%s}}" % self.name
+                    return "{{steps.%s.outputs.artifacts.dflow_bigpar_%s}}" % (self.step_id, self.name)
+                return "{{outputs.artifacts.dflow_bigpar_%s}}" % self.name
         if self.name is not None:
             if self.step_id is not None:
                 return "{{steps.%s.outputs.parameters.%s}}" % (self.step_id, self.name)
@@ -336,40 +359,49 @@ class OutputParameter(ArgoVar):
         return ""
 
     def convert_to_argo(self):
+        description = None
+        if self.type is not None:
+            try:
+                description = jsonpickle.dumps({"type": self.type})
+            except:
+                description = jsonpickle.dumps({"type": None})
+
         if self.save_as_artifact:
             if self.value_from_path is not None:
-                return V1alpha1Artifact(name=self.name, path=self.value_from_path, global_name=self.global_name)
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, path=self.value_from_path, global_name=self.global_name)
             elif self.value_from_parameter is not None:
-                return V1alpha1Artifact(name=self.name, _from=str(self.value_from_parameter), global_name=self.global_name)
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, _from=str(self.value_from_parameter), global_name=self.global_name)
             elif self.value_from_expression is not None:
-                return V1alpha1Artifact(name=self.name, from_expression=str(self.value_from_expression), global_name=self.global_name)
+                return V1alpha1Artifact(name="dflow_bigpar_" + self.name, from_expression=str(self.value_from_expression), global_name=self.global_name)
             else:
                 raise RuntimeError("Not supported.")
+
         if self.value_from_path is not None:
-            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(path=self.value_from_path, default=self.default), global_name=self.global_name)
+            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(path=self.value_from_path, default=self.default), global_name=self.global_name, description=description)
         elif self.value_from_parameter is not None:
-            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(parameter=str(self.value_from_parameter), default=self.default), global_name=self.global_name)
+            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(parameter=str(self.value_from_parameter), default=self.default), global_name=self.global_name, description=description)
         elif self.value_from_expression is not None:
-            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(expression=str(self.value_from_expression), default=self.default), global_name=self.global_name)
+            return V1alpha1Parameter(name=self.name, value_from=V1alpha1ValueFrom(expression=str(self.value_from_expression), default=self.default), global_name=self.global_name, description=description)
         else:
             raise RuntimeError("Output parameter %s is not specified" % self)
 
 class OutputArtifact(ArgoVar):
+    """
+    Output artifact for OP template
+
+    Args:
+        path: path of the output artifact in the container
+        _from: the artifact is from another artifact
+        name: name of the output artifact
+        step_id:
+        type: artifact type
+        save: place to store the output artifact instead of default storage, can be a list
+        archive: compress format of the artifact, None for no compression
+        global_name: global name of the artifact within the workflow
+        from_expression: the artifact is from an expression
+    """
     def __init__(self, path=None, _from=None, name=None, step_id=None, type=None, save=None, archive="tar", global_name=None,
             from_expression=None):
-        """
-        Output artifact for OP template
-        :param path: path of the output artifact in the container
-        :param _from: the artifact is from another artifact
-        :param name: name of the output artifact
-        :param step_id:
-        :param type: artifact type
-        :param save: place to store the output artifact instead of default storage, can be a list
-        :param archive: compress format of the artifact, None for no compression
-        :param global_name: global name of the artifact within the workflow
-        :param from_expression: the artifact is from an expression
-        :return:
-        """
         self.path = path
         self._from = _from
         self.name = name
