@@ -1,3 +1,3 @@
 from .op import OP
-from .opio import OPIOSign, OPIO, Artifact
+from .opio import OPIOSign, OPIO, Artifact, BigParameter
 from .python_op_template import PythonOPTemplate, Slices, TransientError, FatalError, upload_packages
\ No newline at end of file
